/*
 * File: timer.h
 *
 * Contains: KoliadaES TIMER API
 *
 * Edition: TINY
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2015 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __TIMER_H
#define __TIMER_H

// TIMER objects inherit from EVENT
#include "event.h"

#define kTimerType "timer"
#ifndef __TIMER_STRUCT_H
	// external
	typedef class timer;
	#define kTimerClass ((timer)kTimerType)
#else
	// internal (avoid name collision)
	typedef class_type timer_type;
	#define kTimerClass ((timer_type)kTimerType)
#endif
#define kTimerObjectSize (kEventObjectSize + 5)	// timer only words (>excluding< object bit struct)

#define DefineTimer(t) DefineObject(TIMER, kTimerType, kTimerObjectSize, t)
#define ExternTimer(t) ExternObject(TIMER, t)

// TimerTypes (deliberately not an enum)
#define kIntervalTimer   0	// recurring coundown timer
#define kSingleShotTimer 1	// single shot timer (timer stops after firing)

///////////////////////////////////////////////////////////////////////////////////////
//
// timer inheritance info
//
///////////////////////////////////////////////////////////////////////////////////////

#define TIMER_HEADER \
	word __timer_private_data[kTimerObjectSize - kObjectSize];	/* private - excludes BITS */

#define TIMER_BITS	\
	EVENT_BITS \
	word __timer_private_bits :3;/* timer : private  */\

// timer API indices
enum
	{
	__cmStartTimer = __eventMaxApiIndex,
	__cmStopTimer,
	__cmSetTimer,
	__cmSetTicks,
	__cmSetCount,
	__cmPostTimerAt,
	__cmSkewTimer,
	__cmDilateTimer,

	__timerMaxApiIndex,
	};

// timer Alerts
enum
	{
	__no_timer_alerts = 1,
	//
	kMaxTimerError
	};

///////////////////////////////////////////////////////////////////////////////////////
//
// timer API
//
///////////////////////////////////////////////////////////////////////////////////////

// all events have the time they were last fired as an event attribute;
//    - getEventTime(EVENT) returns the time the event last fired
//    - getEventDelta(EVENT) returns the time delta between last and previous fires
// (timer inherits from EVENT)
//
// use;
//		objectCreate(timer, mode, interval);
// to create a timer
//

#ifndef __TickCount__
	#define __TickCount__
	typedef unsigned long TickCount;
#endif
#ifndef __ClockCount__
	#define __ClockCount__
	typedef unsigned long ClockCount;
#endif

// delay n microseconds via a spin loop
// for MSP430!
#define _TargetDelayU(usec)\
	__do(unsigned _usec=usec; _usec >>= 1; while(_usec--) {nop(); nop(); nop();})

#if 0
STATUS cmDelayT(TickCount ticks);
STATUS cmDelayU(ClockCount usec);
STATUS cmDelayM(ClockCount msec);
STATUS cmDelayS(ClockCount secs);

// used to measure elapsed time
#define    cmGetClockT()\
			  cmGetCountT()	// get system tick counter
ClockCount cmGetClockU();	// get system clock micro-seconds
ClockCount cmGetClockM();	// get system clock milli-seconds
ClockCount cmGetClockS();	// get system clock seconds

TickCount cmGetTickRate();	// get the system tick rate (ticks per second)
TickCount cmGetCountT();	// get system tick counter
ClockCount cmGetCountU();	// get system micro-second counter
ClockCount cmGetCountM();	// get system milli-second counter
ClockCount cmGetCountS();	// get system second counter
#endif

STATUS cmTimerExpired(TIMER);					//

STATUS cmStartTimer(TIMER, ...);				// start an interval timer with post args
STATUS cmStopTimer(TIMER);						// stop an interval timer

STATUS cmSetArgs(TIMER, ...);					// set an interval timer post args

STATUS cmSetTimer(TIMER, TickCount);		// set the value of, and start, an interval timer
STATUS cmSetTimerGT(TIMER, TickCount);		// set the value of, and start, an interval timer (if ticks > value)
STATUS cmSetTimerLT(TIMER, TickCount);		// set the value of, and start, an interval timer (if ticks < value)

STATUS cmSetTicks(TIMER, TickCount);		// set time to fire & start (independent of reset count)
STATUS cmSetTicksGT(TIMER, TickCount);		// set time to fire & start if ticks > value (independent of reset count)
STATUS cmSetTicksLT(TIMER, TickCount);		// set time to fire & start if ticks < value (independent of reset count)

STATUS cmGetTimer(TIMER, TickCount *);		// get the value of an interval timer
STATUS cmGetTicks(TIMER, TickCount *);		// get time to fire (ticks from now)

STATUS cmSetCount(TIMER, TickCount);		// set the interval timer reset count (does not start)
STATUS cmGetCount(TIMER, TickCount *);		// get the interval timer reset value (as ticks)

STATUS cmPostTimerAt(TIMER, ClockCount, ...);	// post a timer at a specific time with post args
STATUS cmSkewTimer(TIMER, int);				// add/sub clock skew value (added to remain only)
STATUS cmDilateTimer(TIMER, int);			// add/sub clock dilation value (added to interval and remain)

STATUS cmDelayMs(word ms);						// delay for ms milliseconds
STATUS cmDelayS(word s);						// delay for s seconds

ClockCount cmLapsedTime(ClockCount c1, ClockCount c2);

void cmShowTimers();								// show all timers
word cmTimersPending();							// # timers pending

// system time parameters - exported into link file
extern unsigned const tickRate;
extern volatile ClockCount cmClock;		// free running tick clock
extern volatile UInt32 boot_time;		// seconds since 01/01/93 @ boot (or zero)
extern volatile UInt32 real_time;		// seconds since boot time
extern volatile TickCount real_tick;	//

// all timer functions use ticks but systems relate to ms
#define TICKS(x) ((((TickCount)tickRate)*(x)+500)/1000)			// ms -> ticks
#define MS(x)    ((((TickCount)x)*1000+tickRate/2)/tickRate)	// ticks -> ms

#endif//__TIMER_H
