/*
 * File: except.h
 *
 * Contains: Koliada Exceptions
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __EXCEPT_H
#define __EXCEPT_H
#include <setjmp.h>

//#define EX_TRACE 1

#ifndef USE_EXSTATIC
	#define USE_EXSTATIC 1	// use static jmp_bufs (not re-entrant)!
#endif
#ifndef USE_EXCOOKIE
	#define USE_EXCOOKIE 0	// use cookie in ExContext
#endif

typedef const char *exception;
typedef struct context
	{
	short level;				/* exception context level */
	jmp_buf jump;				/* exception context jump buffer */
	exception magic;			/* exception magic number */
	exception state;			/* current exception state */
	exception signal;			/* current exception signal */
	struct context *prev;	/* previous exception context frame */
#if USE_EXCOOKIE
	void *cookie;				/* current exception cookie */
#endif
	} EXCEPTION_FRAME;
#if USE_EXSTATIC
	#undef STATIC
	#define STATIC static	// static jmp_bufs
#else
	#undef STATIC
	#define STATIC 			// stackd jmp_bufs
#endif

#ifdef MAIN
#ifdef __STDC__
/* ansii declaration */
#define DEFINE_EXCEPTION(NAME) exception NAME = #NAME;
#else
#define DEFINE_EXCEPTION(NAME) exception NAME = "NAME";
#endif
#else
#define DEFINE_EXCEPTION(NAME) exception NAME = #NAME
#define EXTERN_EXCEPTION(NAME) extern exception NAME
#endif

/* some default exceptions */
//extern exception EX_JUMP;	/* jump to explicit frame */
extern const exception EX_BREAK;		/* break from current exception frame */
extern const exception EX_REPEAT;	/* repeat current exception frame */
extern const exception EX_ESCAPE;	/* escape from current exception frame */

#if 0
/* for the sequencing system */
extern exception BACK_SEQ;
extern exception SKIP_SEQ;
extern exception POP_SEQ;
#endif

#ifdef EX_TRACE
void exc_raise(exception s, char *name, unsigned line);
void exc_signal(exception s, char *name, unsigned line);
//void exc_jump(frame, s, name, word line);						/* jump to an exception context frame */
void exc_punt(char *name, unsigned line);
void exc_begin(EXCEPTION_FRAME *, char *name, unsigned line);
unsigned exc_catch(char *name, unsigned line);					/* catch all exceptions here */
unsigned exc_case(exception s, char *name, unsigned line);	/* check for named exception case */
unsigned exc_default(char *name, unsigned line);				/* catch any missed exceptions */
void exc_break(char *name, unsigned line);						/* break out of the context frame handlers */
void exc_end(char *name, unsigned line);							/* end an exception frame */

#define Throw(s)	exc_raise(s, __FILE__, __LINE__)				/* raise an exception */
#define Raise(s)	exc_raise(s, __FILE__, __LINE__)				/* raise an exception */
#define Signal(s)	exc_signal(s, __FILE__, __LINE__)			/* signal an exception */
#define Punt()		exc_punt(__FILE__, __LINE__)					/* pass current exception on */

//#define Jump(f,s)	exc_jump(f,s, __f, __LINE__)				/* jump to a frame exception */
#define SetJump(p)	((p)=&ExFrame)									/* set a jump frame pointer */

/* start exception frame */
#define BEGIN {\
EXCEPTION_FRAME ExFrame;unsigned __exc;char *__f=__FILE__;\
exc_begin(&ExFrame,__f,__LINE__);__exc=setjmp(ExFrame.jump);\
if(__exc==0||__exc==1){
/* start normal code */

/* end normal code - handle exceptions */
#define EXCEPTION exc_raise(EX_BREAK,__f,__LINE__);}else{\
if(exc_catch(__f,__LINE__)){
/* start exception "catch code" */

/* handle NAMEd exception */
#define CATCH(NAME) }if(exc_case(NAME,__f,__LINE__)){\
/* start specific exception code */

/* break from exception handlers */
#define BREAK exc_break(__f,__LINE__);}{

/* handle all other exceptions */
#define DEFAULT }if(exc_default(__f,__LINE__)){\

/* end of exception frame */
#define END }}exc_end(__f,__LINE__);}

#if UES_EXCOOKIE
void *exSetArg(void *);						/* set the exception cookie */
void *exGetArg();								/* get the exception cookie */
#endif

#else // EX_TRACE

void exc_raise(exception s);
void exc_signal(exception s);
//void exc_jump(frame, s);					/* jump to an exception context frame */
void exc_punt();
void exc_begin(EXCEPTION_FRAME *frame);
unsigned exc_catch();						/* catch all exceptions here */
unsigned exc_case(exception s);			/* check for named exception case */
unsigned exc_default();						/* catch any missed exceptions */
void exc_break();								/* break out of the context frame handlers */
void exc_end();								/* end an exception frame */

#define Throw(s)		exc_raise(s)		/* raise an exception */
#define Raise(s)		exc_raise(s)		/* raise an exception */
#define Signal(s)		exc_signal(s)		/* signal an exception */
#define Punt()			exc_punt()			/* pass current exception on */

#define Jump(f,s)		exc_jump(f,s)		/* jump to a frame exception */
#define SetJump(p)	((p)=&ExFrame)		/* set a jump frame pointer */

/* start exception frame */
#define BEGIN {\
EXCEPTION_FRAME ExFrame;unsigned __exc;\
exc_begin(&ExFrame);__exc=setjmp(ExFrame.jump);\
if(__exc==0||__exc==1){
/* start normal code */

/* end normal code - handle exceptions */
#define EXCEPTION exc_raise(EX_BREAK);}else{\
if(exc_catch()){
/* start exception "catch code" */

/* handle NAMEd exception */
#define CATCH(NAME) }if(exc_case(NAME)){\
/* start specific exception code */

/* break from exception handlers */
#define BREAK exc_break();}{

/* handle all other exceptions */
#define DEFAULT }if(exc_default()){\

/* end of exception frame */
#define END }}exc_end();}

#if UES_EXCOOKIE
void *exSetArg(void *);						/* set the exception cookie */
void *exGetArg();								/* get the exception cookie */
#endif

// throw an exception (raise with args)
void throw(exception e, int err, const char *fmt, ...);

#endif//EX_TRACE
#endif//__EXCEPT_H
