/*
 * File: Koliada - system.h
 *
 * Contains: Koliada System API
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under license from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __SYSTEM_H
#define __SYSTEM_H
#include "Koliada.h"

// formatted output
int sysDebug(const char *fmt, ...);		// to the debug device
int sysError(const char *fmt, ...);		// to the error device
int sysPrint(const char *fmt, ...);		// to the user device
int sysTrace(const char *fmt, ...);		// to the trace device

// key input
int sysGetkey(word timeout);			// If a key is hit within timeout (system ticks), return the key value, otherwise -1
												// NOTE: it is >very< likely that I will change delay to milliseconds rather than system ticks (sometime)
int sysHitkey();							// Was a key hit? If so, return the key value leaving it in the queue, otherwise -1

STATUS sysGetXcptionHandler();		// returns a system exception handler function
STATUS sysSetXcptionHandler();		// replaces a system exception handler.

STATUS sysGetInterruptFunctions(InterruptHandler *, InterruptHandler *);	// gets the system interrupt entry/exit functions
STATUS sysSetInterruptFunctions(InterruptHandler, InterruptHandler);			// sets the system interrupt entry/exit functions

STATUS sysInterruptsOn();				// turns on system interrupts		(generally, DO NOT USE!)
STATUS sysInterruptsOff();				// turns off system interrupts	(generally, DO NOT USE!)

STATUS sysGetIntHandler(unsigned ivec, InterruptHandler *fn);	// returns an interrupt handler function
STATUS sysSetIntHandler(unsigned ivec, InterruptHandler fn);	// sets an interrupt handler function
STATUS sysInterruptEnable(unsigned ivec);			// enables interrupts for a particular device vector
STATUS sysInterruptDisable(unsigned ivec);		// disables interrupts for a particular device vector

#if USE_SHARED_COMPONENTS

STATUS sysGetStdioFunction(int fd, SHARED *fn); // STDIO filedes defined in fcntl.h
STATUS sysSetStdioFunction(int fd, SHARED fn);	//

#endif// USE_SHARED_COMPONENTS

STATUS sysGetTickHandler(InterruptHandler *fn, word *arg, word *rate);
STATUS sysSetTickHandler(InterruptHandler  fn, word arg, word rate);

word sysGetTicks();						// returns the current system tick count
word sysGetCount();						// returns the current hardware clock counter
word sysGetCountUs();					// returns the current microsecond clock counter
word sysGetCountMs();					// returns the current millisecond clock counter

void sysDelayUs(word);				// delay for microseconds
void sysDelayMs(word);				// delay for milliseconds
void sysDelayS(word);				// delay for seconds

STATUS sysICacheEnable();				// enables the instruction cache
STATUS sysICacheDisable();				// disables the instruction cache
STATUS sysDCacheEnable();				// enable the data cache
STATUS sysDCacheDisable();				// disable the data cache
STATUS sysCacheEnable();				// enable the both instruction & data cache
STATUS sysCacheDisable();				// disable the both instruction & data cache
STATUS sysICacheInvalidate();			// invalidate the instruction cache
STATUS sysICacheInvalidateRegion(void *, unsigned);
												// invalidate the instruction cache over a memory region
STATUS sysDCacheWriteback();			// writeback the data cache
STATUS sysDCacheWritebackRegion(void *, unsigned);
												// writeback the data cache over a memory region

STATUS sysGpioInit(word n, word mode, ...);
STATUS sysGpioGet(word n, word *set);
STATUS sysGpioSet(word n, word  set);
STATUS sysGpioToggle(word n);
STATUS sysGpioPulse(word n, word d);

void sysLedPulse(word n, word d);	// pulses nth system LED with pulse duration d
void sysLedSet(word n, bool on);		// sets nth system LED on or off
void sysLedToggle(word n);				// toggles nth system LED from its previous state

void sysFatal(char *msg);				// terminate with an error message

#if USE_SHARED_COMPONENTS

STATUS sysDeviceRegister(DeviceId, DeviceDesc, LIBRARY);
																// registers a device management api
STATUS sysDeviceWakeup();								// request that a specific device wake up
STATUS sysAddDeviceSleepCallback(DeviceId, CALLBACK);
																// add a callback to the device sleep timeout queue
STATUS sysSubDeviceSleepTimer(DeviceId, word *ms, CALLBACK *);
																// remove a callback to the device sleep timeout queue
STATUS sysGetDeviceSleepTimer(DeviceId, word *ms, CALLBACK *);
																// get the sleep timeout for a given device
#endif// USE_SHARED_COMPONENTS

STATUS sysReset();										// do a system reset
STATUS sysWakeup();										// wake from low power mode (on)
STATUS sysShutdown();									// shutdown to lowest power mode (off)
STATUS sysSetShutdownTimer(word ms, OBJECT);		// set the system system shutdown timer
STATUS sysGetShutdownTimer(word *ms, OBJECT *);	// get the system system shutdown timer

//
// >danger< there be dragons below...
//
word getSharedHandler(word n);						// returns the current value of spec'd shared handler
word setSharedHandler(word n, HANDLER fn);		// sets current value of spec'd shared handler, returning previous

word getMultixHandler(word n);						// returns the current value of spec'd multix handler
word setMultixHandler(word n, HANDLER fn);		// sets current value of spec'd multix handler, returning previous

void dump(void *mem, unsigned len, unsigned align);	// dump memory to _STDOUT

#endif //__SYSTEM_H
