// MS8607 PTH sensor driver

#include "MS8607.h"
#include "i2c.h"
#include "pindefs.h"

// MS8607 I2C addresses
#define MS8607_PT_ADDRESS      0x76
#define MS8607_RH_ADDRESS      0x40

#define S_REG_LEN                  2
#define DATA_LEN                   3

/* Sensor bit values for self-test and DCDC conbtrol */
#define ST_IRTEMP                             0x01
#define ST_HUMID                              0x02
#define ST_ACC                                0x08
#define ST_GYRO                               0x20

// Internal commands
#define MS8607_CMD_TEMP_T_H         0xE3 // command trig. temp meas. hold master
#define MS8607_CMD_HUMI_T_H         0xE5 // command trig. humidity meas. hold master
#define MS8607_CMD_TEMP_T_NH        0xF3 // command trig. temp meas. no hold master
#define MS8607_CMD_HUMI_T_NH        0xF5 // command trig. humidity meas. no hold master
#define MS8607_CMD_WRITE_U_R        0xE6 // command write user register
#define MS8607_CMD_READ_U_R         0xE7 // command read user register
#define MS8607_CMD_SOFT_RST         0xFE // command soft reset

#define HUMIDITY				           0x01
#define TEMPERATURE			           0x00

#define USR_REG_MASK               0x38  // Mask off reserved bits (3,4,5)
#define USR_REG_DEFAULT            0x02  // Disable OTP reload
#define USR_REG_RES_MASK           0x7E  // Only change bits 0 and 7 (meas. res.)
#define USR_REG_11BITRES           0x81  // 11-bit resolution

/* ------------------------------------------------------------------------------------------------
*                                           Type Definitions
* ------------------------------------------------------------------------------------------------
*/

/* ------------------------------------------------------------------------------------------------
*                                           Local Functions
* ------------------------------------------------------------------------------------------------
*/
static void HalHumiSelect(void);
static bool HalHumiReadData(UInt8 *pBuf, UInt8 nBytes);
static bool HalHumiWriteCmd(UInt8 cmd);

/* ------------------------------------------------------------------------------------------------
*                                           Local Variables
* ------------------------------------------------------------------------------------------------
*/
static UInt8 usr;                         // Keeps user register value
static UInt8 buf[6];                      // Data buffer
static bool  success;
// from hal_sensor.c
static UInt8 buffer[24];
ExternTimer(humiDelayTimer);
/**************************************************************************************************
* @fn          HalHumiInit
*
* @brief       Initialise the humidity sensor driver
*
* @return      none
**************************************************************************************************/
void phtInit(void)
	{
	GPIO.SET(PTH_ON, 1);	// power the sensor
	HalHumiSelect();		// set up I2C

	HalHumiWriteCmd(MS8607_CMD_SOFT_RST);
	cmDelayMs(15);

	// Set 11 bit resolution
	HalSensorReadReg(MS8607_CMD_READ_U_R, &usr, 1);
	usr &= USR_REG_RES_MASK;
	usr |= USR_REG_11BITRES;
	HalSensorWriteReg(MS8607_CMD_WRITE_U_R, &usr, 1);
	}


/**************************************************************************************************
* @fn          HalHumiExecMeasurementStep
*
* @brief       Execute measurement step
*
* @return      none
*/
bool HalHumiExecMeasurementStep(UInt8 state)
	{
	HalHumiSelect();

	switch (state)
		{
		case 0:
				{
				// Start temperature read
				//success = HalHumiWriteCmd(MS8607_CMD_TEMP_T_NH);
				success = HalHumiWriteCmd(MS8607_CMD_TEMP_T_H);
				break;
				}

		case 1:
				{
				// Read and store temperature value
				if (success)
					{
					success = HalHumiReadData(buf, DATA_LEN);

					// Start for humidity read
					if (success)
						{
						//success = HalHumiWriteCmd(MS8607_CMD_HUMI_T_NH);
						success = HalHumiWriteCmd(MS8607_CMD_HUMI_T_H);
						}
					}
				break;
				}

		case 2:
				{
				// Read and store humidity value
				if (success)
					{
					success = HalHumiReadData(buf + DATA_LEN, DATA_LEN);
					HalHumiWriteCmd(MS8607_CMD_SOFT_RST);
					cmDelayMs(15);
					}

				break;
				}
		}

	return success;
	}


/**************************************************************************************************
* @fn          HalHumiReadMeasurement
*
* @brief       Get humidity sensor data
*
* @return      none
*/
bool HalHumiReadMeasurement(UInt8 *pBuf)
	{
	// Store temperature
	pBuf[0] = buf[1];
	pBuf[1] = buf[0];

	// Store humidity
	pBuf[2] = buf[4];
	pBuf[3] = buf[3];

	return success;
	}



/**************************************************************************************************
* @fn          HalHumiTest
*
* @brief       Humidity sensor self test
*
* @return      none
**************************************************************************************************/
/*
bool HalHumiTest(void)
{
  UInt8 val;

  HalHumiSelect();

  // Verify default value
  ST_ASSERT(HalSensorReadReg(MS8607_CMD_READ_U_R,&val,1));
  ST_ASSERT(val==usr);

  return TRUE;
}
*/

/* ------------------------------------------------------------------------------------------------
*                                           Private functions
* -------------------------------------------------------------------------------------------------
*/

/**************************************************************************************************
* @fn          HalHumiSelect
*
* @brief       Select the humidity sensor on the I2C-bus
*
* @return
*/
static void HalHumiSelect(void)
	{
	//Set up I2C that is used to communicate with MS8607
	HalI2CInit(MS8607_RH_ADDRESS, i2cClock_267KHZ);
	}


/**************************************************************************************************
* @fn          halHumiWriteCmd
*
* @brief       Write a command to the humidity sensor
*
* @param       cmd - command to write
*
* @return      TRUE if the command has been transmitted successfully
**************************************************************************************************/
static bool HalHumiWriteCmd(UInt8 cmd)
	{
	/* Send command */
	return HalI2CWrite(1, &cmd) == 1;
	}


/**************************************************************************************************
* @fn          HalHumiReadData
*
* @brief       This function implements the I2C protocol to read from the MS8607.
*
* @param       pBuf - pointer to buffer to place data
*
* @param       nBytes - number of bytes to read
*
* @return      TRUE if the required number of bytes are received
**************************************************************************************************/
static bool HalHumiReadData(UInt8 *pBuf, UInt8 nBytes)
	{
	/* Read data */
	return HalI2CRead(nBytes, pBuf) == nBytes;
	}

/*  Conversion algorithm, humidity
 */
double calcHumRel(UInt16 rawH)
	{
	double v;

	rawH &= ~0x0003; // clear bits [1..0] (status bits)
	//-- calculate relative humidity [%RH] --
	v = (-7.0 + 125.0 / 65536 * (double)rawH) * 10; // RH= -6 + 125 * SRH/2^16

	return v;
	}

/*  Conversion algorithm, temperature
*/
double calcHumTmp(UInt16 rawT)
	{
	double v;
	//rawT &= ~0x0003;
	//-- calculate temperature [C] --46.85
	v = (-46.85 + 175.72 / 65536 * (double)(UInt16)rawT) * 100;

	return v;
	}


/**************************************************************************************************
 * @fn          HalSensorReadReg
 *
 * @brief       This function implements the I2C protocol to read from a sensor. The sensor must
 *              be selected before this routine is called.
 *
 * @param       addr - which register to read
 * @param       pBuf - pointer to buffer to place data
 * @param       nBytes - numbver of bytes to read
 *
 * @return      TRUE if the required number of bytes are reveived
 **************************************************************************************************/
bool HalSensorReadReg(UInt8 addr, UInt8 *pBuf, UInt8 nBytes)
	{
	UInt8 i = 0;

	/* Send address we're reading from */
	if (HalI2CWrite(1, &addr) == 1)
		{
		/* Now read data */
		i = HalI2CRead(nBytes, pBuf);
		}

	return i == nBytes;
	}

/**************************************************************************************************
* @fn          HalSensorWriteReg
* @brief       This function implements the I2C protocol to write to a sensor. he sensor must
*              be selected before this routine is called.
*
* @param       addr - which register to write
* @param       pBuf - pointer to buffer containing data to be written
* @param       nBytes - number of bytes to write
*
* @return      TRUE if successful write
*/
bool HalSensorWriteReg(UInt8 addr, UInt8 *pBuf, UInt8 nBytes)
	{
	UInt8 i;
	UInt8 *p = buffer;

	/* Copy address and data to local buffer for burst write */
	*p++ = addr;
	for (i = 0; i < nBytes; i++)
		{
		*p++ = *pBuf++;
		}
	nBytes++;

	/* Send address and data */
	i = HalI2CWrite(nBytes, buffer);
	if (i != nBytes)
		{
		// HAL_TOGGLE_LED2();
		}
	return (i == nBytes);
	}


/*********************************************************************
*********************************************************************/
