/**************************************************************************************************
  Filename:       hal_i2c.c
  Revised:        $Date: 2012-09-21 06:30:38 -0700 (Fri, 21 Sep 2012) $
  Revision:       $Revision: 31581 $

  Description:    This module defines the HAL I2C API for the CC2541ST. It
                  implements the I2C master.


**************************************************************************************************/

/* ------------------------------------------------------------------------------------------------
 *                                          Includes
 * ------------------------------------------------------------------------------------------------
 */

#include "i2c.h"

/* ------------------------------------------------------------------------------------------------
 *                                          Constants
 * ------------------------------------------------------------------------------------------------
 */
#define I2C_ENS1            BIT6
#define I2C_STA             BIT5
#define I2C_STO             BIT4
#define I2C_SI              BIT3
#define I2C_AA              BIT2
#define I2C_MST_RD_BIT      BIT0  // Master RD/WRn bit to be OR'ed with Slave address.

#define I2C_CLOCK_MASK      0x83

#define I2C_PXIFG           P2IFG
#define I2C_IF              P2IF
#define I2C_IE              BIT1

/* ------------------------------------------------------------------------------------------------
 *                                           Typedefs
 * ------------------------------------------------------------------------------------------------
 */

typedef enum
	{
	// HAL_I2C_MASTER mode statuses.
	mstStarted   = 0x08,
	mstRepStart  = 0x10,
	mstAddrAckW  = 0x18,
	mstAddrNackW = 0x20,
	mstDataAckW  = 0x28,
	mstDataNackW = 0x30,
	mstLostArb   = 0x38,
	mstAddrAckR  = 0x40,
	mstAddrNackR = 0x48,
	mstDataAckR  = 0x50,
	mstDataNackR = 0x58,
	} i2cStatus_t;

/* ------------------------------------------------------------------------------------------------
 *                                           Macros
 * ------------------------------------------------------------------------------------------------
 */

#define I2C_WRAPPER_DISABLE() st( I2CWC    =     0x00;              )
#define I2C_CLOCK_RATE(x)     st( I2CCFG  &=    ~I2C_CLOCK_MASK;    \
                                  I2CCFG  |=     x;                 )
#define I2C_SET_NACK()        st( I2CCFG &= ~I2C_AA; )
#define I2C_SET_ACK()         st( I2CCFG |=  I2C_AA; )

// Enable I2C bus
#define I2C_ENABLE()          st( I2CCFG |= (I2C_ENS1); )
#define I2C_DISABLE()         st( I2CCFG &= ~(I2C_ENS1); )

// Must clear SI before setting STA and then STA must be manually cleared.
#define I2C_STRT() st (             \
                                    I2CCFG &= ~I2C_SI;                \
                                    I2CCFG |= I2C_STA;                \
                                    while ((I2CCFG & I2C_SI) == 0);   \
                                    I2CCFG &= ~I2C_STA; \
                      )

// Must set STO before clearing SI.
#define I2C_STOP() st (             \
                                    I2CCFG |= I2C_STO;                \
                                    I2CCFG &= ~I2C_SI;                \
                                    while ((I2CCFG & I2C_STO) != 0);  \
                      )

// Stop clock-stretching and then read when it arrives.
#define I2C_READ(_X_) st (          \
                                    I2CCFG &= ~I2C_SI;                \
                                    while ((I2CCFG & I2C_SI) == 0);   \
                                    (_X_) = I2CDATA;                  \
                         )

// First write new data and then stop clock-stretching.
#define I2C_WRITE(_X_) st (         \
                                    I2CDATA = (_X_);                  \
                                    I2CCFG &= ~I2C_SI;                \
                                    while ((I2CCFG & I2C_SI) == 0);   \
                          )


/* ------------------------------------------------------------------------------------------------
 *                                       Local Variables
 * ------------------------------------------------------------------------------------------------
 */
static UInt8 i2cAddr;  // Target Slave address pre-shifted up by one leaving RD/WRn LSB as zero.


/*void I2C_WRITE(UInt8 buf)
{
  I2CDATA = buf;
  I2CCFG &= ~I2C_SI;
  while ((I2CCFG & I2C_SI) == 0);
}

UInt8 I2C_READ(UInt8 buf)
{
  I2CCFG &= ~I2C_SI;
  while ((I2CCFG & I2C_SI) == 0);
  buf = I2CDATA;
  return buf;
}*/
/**************************************************************************************************
 * @fn          i2cMstStrt
 *
 * @brief       Attempt to send an I2C bus START and Slave Address as an I2C bus Master.
 *
 * input parameters
 *
 * @param       RD_WRn - The LSB of the Slave Address as Read/~Write.
 *
 * @return      The I2C status of the START request or of the Slave Address Ack.
 */
static UInt8 i2cMstStrt(UInt8 RD_WRn)
	{
	I2C_STRT();
	//I2CCFG &= ~I2C_SI;
	//2CCFG |= I2C_STA;
	//while ((I2CCFG & I2C_SI) == 0);
	//I2CCFG &= ~I2C_STA;

	if (I2CSTAT == mstStarted) /* A start condition has been transmitted */
		{
		I2C_WRITE(i2cAddr | RD_WRn);
		}

	return I2CSTAT;
	}

/**************************************************************************************************
 * @fn          HalI2CInit
 *
 * @brief       Initialize the I2C bus as a Master.
 *
 * input parameters
 *
 * @param       address - I2C slave address.
 * @param       clockRate - I2C clock rate.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
void HalI2CInit(UInt8 address, i2cClock_t clockRate)
	{
	i2cAddr = address << 1;

	I2C_WRAPPER_DISABLE();
	//I2CWC    =     0x00;
	//I2CADDR = 0; // no multi master support at this time
	I2C_CLOCK_RATE(clockRate);
	//I2CCFG  &=    ~I2C_CLOCK_MASK;
	//I2CCFG  |=     clockRate;
	I2C_ENABLE();
	//I2CCFG |= (I2C_ENS1);
	}

/**************************************************************************************************
 * @fn          HalI2CRead
 *
 * @brief       Read from the I2C bus as a Master.
 *
 * input parameters
 *
 * @param       len - Number of bytes to read.
 * @param       pBuf - Pointer to the data buffer to put read bytes.
 *
 * output parameters
 *
 * None.
 *
 * @return      The number of bytes successfully read.
 */
UInt8 HalI2CRead(UInt8 len, UInt8 *pBuf)
	{
	UInt8 cnt = 0;

	if (i2cMstStrt(I2C_MST_RD_BIT) != mstAddrAckR)
		{
		len = 0;
		}

	// All bytes are ACK'd except for the last one which is NACK'd. If only
	// 1 byte is being read, a single NACK will be sent. Thus, we only want
	// to enable ACK if more than 1 byte is going to be read.
	if (len > 1)
		{
		I2C_SET_ACK();
		//I2CCFG |=  I2C_AA;
		}

	while (len > 0)
		{
		// slave devices require NACK to be sent after reading last byte
		if (len == 1)
			{
			I2C_SET_NACK();
			//I2CCFG &= ~I2C_AA;
			}

		// read a byte from the I2C interface
		I2C_READ(*pBuf++);
		cnt++;
		len--;

		if (I2CSTAT != mstDataAckR)
			{
			if (I2CSTAT != mstDataNackR)
				{
				// something went wrong, so don't count last byte
				cnt--;
				}
			break;
			}
		}
	I2C_STOP();
	//I2CCFG |= I2C_STO;
	//I2CCFG &= ~I2C_SI;
// while ((I2CCFG & I2C_STO) != 0);

	return cnt;
	}

/**************************************************************************************************
 * @fn          HalI2CWrite
 *
 * @brief       Write to the I2C bus as a Master.
 *
 * input parameters
 *
 * @param       len - Number of bytes to write.
 * @param       pBuf - Pointer to the data buffer to write.
 *
 * output parameters
 *
 * None.
 *
 * @return      The number of bytes successfully written.
 */
UInt8 HalI2CWrite(UInt8 len, UInt8 *pBuf)
	{
	if (i2cMstStrt(0) != mstAddrAckW)
		{
		len = 0;
		}

	for (UInt8 cnt = 0; cnt < len; cnt++)
		{
		I2C_WRITE(*pBuf++);

		if (I2CSTAT != mstDataAckW)
			{
			if (I2CSTAT == mstDataNackW)
				{
				len = cnt + 1;
				}
			else
				{
				len = cnt;
				}
			break;
			}
		}

	I2C_STOP();
	//I2CCFG |= I2C_STO;
	//I2CCFG &= ~I2C_SI;
	//while ((I2CCFG & I2C_STO) != 0);

	return len;
	}

/**************************************************************************************************
 * @fn          HalI2CDisable
 *
 * @brief       Places the I2C bus in inactive mode
 *
 * input parameters
 *
 * None.
 *
 * output parameters
 *
 * None.
 *
 * @return      None.
 */
void HalI2CDisable(void)
	{
	I2C_DISABLE();
	//I2CCFG &= ~(I2C_ENS1);
	}

/*********************************************************************
*********************************************************************/
