/*
 *	File: BoardConfig.c
 *
 *	Contains: specific configuration for Tenacore TAG
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#include "BoardConfig.h"

#include "build.h"
#include "sdk.h"

// BoardConfig is used to set the contents of a flash block which
// configures the system according to specific board requirements.
//
// Since board config details are stored in a flash block (Binary Configuration
// Block - BCB), they are independent all system code and may be updated at any
// time. This allows modification of system behavior (via the BCB) without
// changing the system binaries and, in turn, (generally) allows the same system
// binaries to be used across multiple board/chip types of the same
// architecture; e.g. MSP430, Stellaris, 8051, etc.

////////////////////////////////////////////////////////////////////////////////
//
// Edit settings & pin descriptor table to match your board/requirements
//
////////////////////////////////////////////////////////////////////////////////

// change board details as required for specific board
#define BOARD_NAME "Tenacore Tag"// 32 char max
#if USE_DEFAULT_BOARD_CONFIG
// default values only - actuals are (typically) in the IDE
#define BOARD_MAJOR  1	// board major version number
#define BOARD_MINOR  0	// board minor version number
#define BOARD_CONFIG 0	// board configuration id
#endif

#define SYSTEM_CONFIG (kUse32kHzXtal)

#define DEBUG_UART  1
#define BAUD_RATE 1996094	// pretty much the fastest Win32 will handle...
//#define BAUD_RATE 921600
//#define BAUD_RATE 115200

// BoardData segment
#pragma constseg=BOARDDATA // board data segment (don't mess with this)!

// Pin Descriptor Table
// describes a mapped 'pin' to the system
// portable access to pins is via the abstracted pin number in this table
// for fastest access, the (non portable) compiler pin names can be used
// this table also used to configure the described pins on startup

const PinDescriptor pinTable[] =
	{
// mark unused entries with GPIOX
// there is an implicit assumption that LEDs 1-4 and SW1-4 are first 8 entries
// single entry can describe multiple pins with same port/config (OR the bits together)
// for outputpins, REN is the init state (lo = 0, 1 = hi)
//                          /------------ dir: 0 = input,   1 = output
//                         /  /---------- ies: 0 = rising,  1 = falling
//                        /  /  /-------- ren: 0 - pull,    1 - 3-state
//                       /  /  /  /------ inv: 0 - normal,  1 - invert
// pin    port   mask   /  /  /  /  /---- set: 0 = off,     1 = on
//-----|-------|-----|-----------------|------------------------------------------
	/*   0 */ {PORT1, BIT0, {1, 0, 0, 0, 0}},	/*< RED, SYSLED >*/
	/*   1 */ {PORT1, BIT1, {1, 0, 0, 0, 0}},	/*< GRN >*/
	/*   2 */ {PORT0, BIT0, {1, 0, 0, 0, 0}},	/*< Photo Power >*/
	/*   3 */ {PORT1, BIT2, {0, 1, 0, 0, 0}},	/*< SW1 >*/
	/*   4 */ {PORT0, BIT7, {0, 0, 1, 0, 0}},	/*< ACC_INT1 >*/
	/*   5 */ {PORT0, BIT6, {0, 0, 1, 0, 0}},	/*< ACC_INT2 >*/
	/*   6 */ {PORT0, BIT5, {1, 0, 0, 0, 0}},	/*< ACC_SCL >*/
	/*   7 */ {PORT0, BIT2, {0, 0, 1, 0, 0}},	/*< ACC_SDO >*/
	/*   8 */ {PORT0, BIT3, {1, 0, 0, 0, 0}},	/*< ACC_SDI >*/
	/*   9 */ {PORT0, BIT4, {1, 0, 0, 0, 0}},	/*< ACC_CS >*/
	/*  10 */ {PORT1, BIT3, {1, 0, 0, 0, 0}},	/*< ACC_ON >*/
	/*  11 */ {PORT0, BIT1, {1, 0, 0, 0, 0}},	/*< Photo Power >*/
	/*  12 */ {PORT2, BIT0, {1, 0, 0, 0, 0}},	/*< PTH_ON >*/
	/*  13 */ {PORT2, BIT2, {1, 0, 0, 0, 0}},	/*< P2_2 >*/
	/*  14 */ {PORT1, BIT4, {1, 0, 0, 0, 0}},	/*< P1_4 >*/
	/*  15 */ {PORT1, BIT5, {1, 0, 0, 0, 0}},	/*< P1_5 >*/
	};

// BoardArgs segment
#pragma constseg=BOARDARGS // board data segment (don't mess with this)!
#pragma data_alignment=16

// args may be used to change the way your system starts

// primary args
const char arg0_0[] = "tag";
const char *const argv0[] = {arg0_0, 0};

// secondary args (used if/when primary app fails/exits)
const char *const argv1[] = { 0 };

// BoardEnvp segment
#pragma constseg=BOARDENVP // board data segment (don't mess with this)!
#pragma data_alignment=16

// name=value pairs may be used to change the way your system runs

// system env buffer
const char envp[] =
	{
	// all but the last name=value pair must be explicitly terminated by and additional "\0"
	// "env1=abc" "\0"
	// "env2=def" "\0"
	// for an empty set use ""
	"emNodeName=" DB_NAME //"\0"
//	""
	};

////////////////////////////////////////////////////////////////////////////////
//
#include "BoardConfigCC8051.h"// must be between config details and BoardConfig()!
//
////////////////////////////////////////////////////////////////////////////////

// BoardConfig() is run once and only when the config page is installed!

int BoardConfig(int argc, char **argv)
	{
	(void) argv;
	(void) argc;

	// here we can do any config related test/debug/config

	// As an example, it can interrogate a server that provides a serial
	// number and/or other unique board config, run calibrations & etc...

#if TEST_BATTERY_ADC
	while (1)
		{
		word adc(byte spec);
		word lowBatteryValue = 444;//Rev 4.2
		P1_2 = 1;// turn on battery sense
		volatile word i;
		for (i = 0; i < 3000; i++); // 'warm-up' delay
		word value = (adc(0x80 | 0x30 | 0x07) >> 3); // P0_7 / AVDD ref
		P1_2 = 0;// turn off battery sense
		byte level = value / lowBatteryValue;
		debug("battery check: %u/4096 - level %u\n", value, level);
		}
#endif

	return 0;// non zero return will start otaLoader (if installed)
	}
